@@ -1,5 +1,6 @@
 import math
 import os
+import re
 import tarfile
 import wave
 
@@ -8,6 +9,8 @@ from PyQt5.QtCore import pyqtSignal, QObject, QDir, Qt
 from PyQt5.QtWidgets import QApplication
 
 import urh.cythonext.signal_functions as signal_functions
+
+from urh import settings
 from urh.ainterpretation import AutoInterpretation
 from urh.signalprocessing.Filter import Filter
 from urh.signalprocessing.IQArray import IQArray
@@ -55,6 +58,7 @@ class Signal(QObject):
         self.iq_array = IQArray(None, np.int8, 1)
 
         self.wav_mode = filename.endswith(".wav")
+        self.flipper_raw_mode = filename.endswith(".sub")
         self.__changed = False
         if modulation is None:
             modulation = "FSK"
@@ -69,13 +73,20 @@ class Signal(QObject):
         if len(filename) > 0:
             if self.wav_mode:
                 self.__load_wav_file(filename)
+            elif self.flipper_raw_mode:
+                self.__load_sub_file(filename)
             elif filename.endswith(".coco"):
                 self.__load_compressed_complex(filename)
             else:
                 self.__load_complex_file(filename)
 
             self.filename = filename
-            self.noise_threshold = AutoInterpretation.detect_noise_level(self.iq_array.magnitudes)
+
+            default_noise_threshold = settings.read("default_noise_threshold", "automatic")
+            if default_noise_threshold == "automatic":
+                self.noise_threshold = AutoInterpretation.detect_noise_level(self.iq_array.magnitudes)
+            else:
+                self.noise_threshold = float(default_noise_threshold) / 100 * self.max_magnitude
         else:
             self.filename = ""
 
@@ -124,6 +135,30 @@ class Signal(QObject):
 
         self.sample_rate = sample_rate
 
+    def __load_sub_file(self, filename: str):
+        # Flipper RAW file format (OOK): space separated values, number of samples above (positive value -> 1)
+        # or below (negative value -> 0) center
+        params = {"min": 0, "max": 255, "fmt": np.uint8}
+        params["center"] = (params["min"] + params["max"]) / 2
+        arr = []
+        with open(filename, 'r') as subfile:
+            for line in subfile:
+                dataline = re.match(r'RAW_Data:\s*([-0-9 ]+)\s*$', line)
+                if dataline:
+                    values = dataline[1].strip().split(' ')
+                    for value in values:
+                        try:
+                            intval = int(value)
+                            if intval > 0:
+                                arr.extend(np.full(intval, params["max"], dtype=params["fmt"]))
+                            else:
+                                arr.extend(np.zeros(-intval, dtype=params["fmt"]))
+                        except ValueError:
+                            logger.warning("Skipped invalid value {0} in sub file {1}.\nLine <{2}>\nValues:<{3}>\n".format(value, filename, line, values))
+        self.iq_array = IQArray(None, np.float32, n=len(arr))
+        self.iq_array.real = np.multiply(1 / params["max"], np.subtract(arr, params["center"]))
+        self.__already_demodulated = True
+
     def __load_compressed_complex(self, filename: str):
         obj = tarfile.open(filename, "r")
         members = obj.getmembers()
@@ -347,6 +382,13 @@ class Signal(QObject):
         except AttributeError:
             return np.zeros(0, dtype=np.float32)
 
+    @property
+    def imag_plot_data(self):
+        try:
+            return self.iq_array.imag
+        except AttributeError:
+            return np.zeros(0, dtype=np.float32)
+
     @property
     def changed(self) -> bool:
         """
@@ -375,9 +417,12 @@ class Signal(QObject):
         QApplication.instance().restoreOverrideCursor()
 
     def quad_demod(self):
-        return signal_functions.afp_demod(self.iq_array.data, self.noise_threshold,
-                                          self.modulation_type, self.modulation_order,
-                                          self.costas_loop_bandwidth)
+        if self.noise_threshold < self.max_magnitude:
+            return signal_functions.afp_demod(self.iq_array.data, self.noise_threshold,
+                                              self.modulation_type, self.modulation_order,
+                                              self.costas_loop_bandwidth)
+        else:
+            return np.zeros(2, dtype=np.float32)
 
     def calc_relative_noise_threshold_from_range(self, noise_start: int, noise_end: int):
         num_digits = 4
@@ -408,8 +453,10 @@ class Signal(QObject):
         new_signal.__bits_per_symbol = self.bits_per_symbol
         new_signal.__center = self.center
         new_signal.wav_mode = self.wav_mode
+        new_signal.flipper_raw_mode = self.flipper_raw_mode
         new_signal.__already_demodulated = self.__already_demodulated
         new_signal.changed = True
+        new_signal.sample_rate = self.sample_rate
         return new_signal
 
     def get_thresholds_for_center(self, center: float, spacing=None):
